==> ix-4.4-src.zip: ix-4.4/development/java/ix/icore/process/PNodeEnd.java <==
/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu Oct 19 05:24:03 2006 by Jeff Dalton
 * Copyright: (c) 2004, 2006, AIAI, University of Edinburgh
 */

package ix.icore.process;

import java.util.*;

import ix.icore.Status;
import ix.icore.HasStatus;
import ix.icore.domain.End;
import ix.icore.domain.NodeEndRef;
import ix.icore.domain.Ordering;

import ix.util.*;
import ix.util.lisp.Lisp;
import ix.util.context.*;

/**
 * One end of a PNode.
 */
public class PNodeEnd extends TimePoint implements HasStatus {

    private End end;
    private PNode node;

    /** Context-dependent status. */
    private ContextValue status =
	new TypedContextValue(Status.class, Status.BLANK);

    /** Contex-dependent list of node-ends linked before this one. */
    private LLQueue predecessors = new LLQueue();

    /** Contex-dependent list of node-ends linked after this one. */
    private LLQueue successors = new LLQueue();

    public PNodeEnd(End end, PNode node) {
	this.end = end;
	this.node = node;
    }

    public static PNodeEnd fromRef(Map idToNodeMap, NodeEndRef ref) {
	PNode node = (PNode)idToNodeMap.get(ref.getNode());
	End end = ref.getEnd();
	Debug.expect(node != null, "Can't find node for", ref);
	if (end == End.BEGIN)
	    return node.getBegin();
	else if (end == End.END)
	    return node.getEnd();
	else
	    throw new ConsistencyException("Bad end in", ref);
    }

    public static void addOrdering(Map idToNodeMap, Ordering ord) {
	PNodeEnd from = fromRef(idToNodeMap, ord.getFrom());
	PNodeEnd to = fromRef(idToNodeMap, ord.getTo());
	from.linkBefore(to);
    }

    public PNode getNode() {
	return node;
    }

    public End getEnd() {
	return end;
    }

    public Status getStatus() {
	return (Status)status.get();
    }

    public void setStatus(Status s) {
	status.set(s);
	PNode.allComputeStatus(getSuccessors());
    }

    public void computeStatus() {
	if (!PNode.computeStatus) return;
	Status stat = getStatus();
	if (stat == Status.BLANK) {
	    if (PNode.allHaveStatus(getPredecessors(), Status.COMPLETE)) {
		setStatus(Status.POSSIBLE);
		if (end == End.BEGIN)
		    node.setStatus(Status.POSSIBLE);
	    }
	}
    }


    public List getPredecessors() {
	return predecessors.contents();
    }

    public List getSuccessors() {
	return successors.contents();
    }

    public void linkBefore(PNodeEnd to) {
	if (to.node != this.node)
	    Debug.noteln("-- " + this + " --> " + to);
	if (!successors.contains(to)) {
	    successors.add(to);
	    to.predecessors.add(this);
	}
    }

    public void linkAfter(PNodeEnd from) {
	if (from.node != this.node)
	    Debug.noteln("-- " + from + " --> " + this);
	if (!from.successors.contains(this)) {
	    from.successors.add(this);
	    predecessors.add(from);
	}
    }

    public void unlink() {
	// Link every predecessor of this node-end before every successor.
	for (Iterator pi = predecessors.iterator(); pi.hasNext();) {
	    PNodeEnd pre = (PNodeEnd)pi.next();
	    for (Iterator si = successors.iterator(); si.hasNext();) {
		PNodeEnd suc = (PNodeEnd)si.next();
		pre.linkBefore(suc);
	    }
	}
	// Tell predecessors and successors to forget their links
	// to this node-end.
	for (Iterator pi = predecessors.iterator(); pi.hasNext();) {
	    PNodeEnd pre = (PNodeEnd)pi.next();
	    pre.successors.remove(this);
	}
	for (Iterator si = successors.iterator(); si.hasNext();) {
	    PNodeEnd suc = (PNodeEnd)si.next();
	    suc.predecessors.remove(this);
	}
    }

    /**
     * Returns a list containing all the node-ends that are forced
     * by ordering constraints to be after this one.
     */
    public List getShadowedEnds() {
	// /\/: It would be better to return a Set, since we
	// usually want the result only so we can see whether
	// various node-ends are in it.  It would also be possible
	// to mark the shadowed node-ends (perhaps by having
	// a mark field in PNodeEnd) and not construct any
	// object that contained all the shadowed ends.
	TopologicalSorter tsort = new TopologicalSorter() {
	    protected Collection getChildren(Object ne) {
		return ((PNodeEnd)ne).getSuccessors();
	    }
	};
	return tsort.sort(Lisp.list(this));
    }

    public String toString() {
	return end + "_of " + node;
    }

}

==> ix-4.5-src.zip: ix-4.5/development/java/ix/icore/process/PNodeEnd.java <==
/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Mon May 21 21:04:20 2007 by Jeff Dalton
 * Copyright: (c) 2004, 2006, 2007, AIAI, University of Edinburgh
 */

package ix.icore.process;

import java.util.*;

import ix.icore.Status;
import ix.icore.HasStatus;
import ix.icore.domain.End;
import ix.icore.domain.NodeEndRef;
import ix.icore.domain.Ordering;

import ix.util.*;
import ix.util.lisp.Lisp;
import ix.util.context.*;

/**
 * One end of a PNode.
 */
public class PNodeEnd extends TimePoint implements HasStatus {

    private End end;
    private PNode node;

    /** Context-dependent status. */
    private ContextValue status =
	new TypedContextValue(Status.class, Status.BLANK);

    /** Contex-dependent list of node-ends linked before this one. */
    private LLQueue predecessors = new LLQueue();

    /** Contex-dependent list of node-ends linked after this one. */
    private LLQueue successors = new LLQueue();

    public PNodeEnd(End end, PNode node) {
	this.end = end;
	this.node = node;
    }

    public static PNodeEnd fromRef(Map idToNodeMap, NodeEndRef ref) {
	PNode node = (PNode)idToNodeMap.get(ref.getNode());
	End end = ref.getEnd();
	Debug.expect(node != null, "Can't find node for", ref);
	if (end == End.BEGIN)
	    return node.getBegin();
	else if (end == End.END)
	    return node.getEnd();
	else
	    throw new ConsistencyException("Bad end in", ref);
    }

    public static void addOrdering(Map idToNodeMap, Ordering ord) {
	PNodeEnd from = fromRef(idToNodeMap, ord.getFrom());
	PNodeEnd to = fromRef(idToNodeMap, ord.getTo());
	from.linkBefore(to);
    }

    public PNode getNode() {
	return node;
    }

    public End getEnd() {
	return end;
    }

    public PNodeEnd getOtherNodeEnd() {
	return end == End.BEGIN ? getNode().getEnd() : getNode().getBegin();
    }

    public Status getStatus() {
	return (Status)status.get();
    }

    public void setStatus(Status s) {
	status.set(s);
	PNode.allComputeStatus(getSuccessors());
    }

    public void computeStatus() {
	if (!PNode.computeStatus) return;
	Status stat = getStatus();
	if (stat == Status.BLANK) {
	    if (PNode.allHaveStatus(getPredecessors(), Status.COMPLETE)) {
		setStatus(Status.POSSIBLE);
		if (end == End.BEGIN)
		    node.setStatus(Status.POSSIBLE);
	    }
	}
    }

    public List getPredecessors() {
	return predecessors.contents();
    }

    public List getSuccessors() {
	return successors.contents();
    }

    public void linkBefore(PNodeEnd to) {
	if (to.node != this.node)
	    Debug.noteln("-- " + this + " --> " + to);
	if (!successors.contains(to)) {
	    successors.add(to);
	    to.predecessors.add(this);
	}
    }

    public void linkAfter(PNodeEnd from) {
	if (from.node != this.node)
	    Debug.noteln("-- " + from + " --> " + this);
	if (!from.successors.contains(this)) {
	    from.successors.add(this);
	    predecessors.add(from);
	}
    }

    public void unlink() {
	// Link every predecessor of this node-end before every successor.
	for (Iterator pi = predecessors.iterator(); pi.hasNext();) {
	    PNodeEnd pre = (PNodeEnd)pi.next();
	    for (Iterator si = successors.iterator(); si.hasNext();) {
		PNodeEnd suc = (PNodeEnd)si.next();
		pre.linkBefore(suc);
	    }
	}
	// Tell predecessors and successors to forget their links
	// to this node-end.
	for (Iterator pi = predecessors.iterator(); pi.hasNext();) {
	    PNodeEnd pre = (PNodeEnd)pi.next();
	    pre.successors.remove(this);
	}
	for (Iterator si = successors.iterator(); si.hasNext();) {
	    PNodeEnd suc = (PNodeEnd)si.next();
	    suc.predecessors.remove(this);
	}
    }

    /**
     * Returns a list containing all the node-ends that are forced
     * by ordering constraints to be after this one.
     */
    public List getShadowedEnds() {
	// /\/: It would be better to return a Set, since we
	// usually want the result only so we can see whether
	// various node-ends are in it.  It would also be possible
	// to mark the shadowed node-ends (perhaps by having
	// a mark field in PNodeEnd) and not construct any
	// object that contained all the shadowed ends.
	TopologicalSorter tsort = new TopologicalSorter() {
	    protected Collection getChildren(Object ne) {
		return ((PNodeEnd)ne).getSuccessors();
	    }
	};
	return tsort.sort(Lisp.list(this));
    }

    public String toString() {
	return end + "_of " + node;
    }

}
